package com.iden.bms.face;

import com.face.monitor.FaceMonitor;
import com.face.monitor.model.FaceModel;
import com.face.monitor.model.FaceRecogRetrieveResult;
import com.face.monitor.model.Image;
import com.face.monitor.model.RecogResult;
import com.iden.common.util.ByteUtil;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

public class FaceIdenTool {
    private static final Logger logger = LogManager.getLogger(FaceIdenTool.class);

    /**
     * 初始化引擎，加载图像人脸特征库
     * @param idenRoot
     * @param imgFiles
     * @return
     */
    public static FaceMonitor initFaceMonitor(String idenRoot, File[] imgFiles) {
        //初始化引擎
        FaceMonitor faceMonitor = new FaceMonitor();
        faceMonitor.init(idenRoot + "face/model", 0);

        List<byte[]> faceTestImageList = new ArrayList<>();
        for(File imgFile : imgFiles) {
            faceTestImageList.add(readFileBytes(imgFile.getAbsolutePath()));
        }
        FaceModel[] faceModels = faceMonitor.extractFeature(faceTestImageList);
        //加载人脸特征库
        faceMonitor.loadFaceDataset(faceModels);

        return faceMonitor;
    }

    /**
     * 批量获取人脸特征码
     * @param idenRoot
     * @param imgFiles
     * @return
     */
    public static FaceModel[] extractFeature(String idenRoot, File[] imgFiles){
        //初始化引擎
        FaceMonitor faceMonitor = new FaceMonitor();
        faceMonitor.init(idenRoot + "face/model", 0);

        List<byte[]> faceTestImageList = new ArrayList<>();
        for(File imgFile : imgFiles) {
            faceTestImageList.add(readFileBytes(imgFile.getAbsolutePath()));
        }
        FaceModel[] faceModels = faceMonitor.extractFeature(faceTestImageList);
        //释放人脸引擎
        faceMonitor.releaseEngine();
        return faceModels;
    }


    //得到单个特征码
    public static String getFeatPtr(String idenRoot, File imgFile) {
        File[] imgFiles = new File[]{imgFile};
        FaceModel[] faceModels = extractFeature(idenRoot,imgFiles);

        if(faceModels != null && faceModels.length > 0) {
            FaceModel faceModel = faceModels[0];
            logger.info("name; " + faceModel.getName());
            logger.info("personId: " + faceModel.getPersonId());
            return ByteUtil.byte2Hex(faceModel.getFeatValue());
        }
        return null;
    }


    /**
     * faceModel在faceMonitor的人脸特征库中检索命中的index
     * @param faceMonitor
     * @param faceModel
     * @return
     */
    public static int getHitIndex(FaceMonitor faceMonitor, FaceModel faceModel) {
        FaceRecogRetrieveResult faceRecogRetrieveResult = faceMonitor.faceRecogRetrieveFaceDataset(faceModel);
        if(faceRecogRetrieveResult != null) {
            int index = faceRecogRetrieveResult.getOnePredictIds()[0];
            float score = faceRecogRetrieveResult.getOnePredictScores()[0];
            if (score >= 65) {
                return index;
            }
        }
        return -1;
    }

    public static RecogResult watchFrame(String idenRoot,  String dataPath) {
        //初始化引擎
        FaceMonitor faceMonitor = new FaceMonitor();
        faceMonitor.init(idenRoot + "face/model", 0);
        Image image = faceMonitor.readImage(dataPath);
        return faceMonitor.faceRecogMonitor(image);
    }

    //释放人脸引擎
    public static void releaseEngine(FaceMonitor faceMonitor) {
        if(faceMonitor != null){
            faceMonitor.releaseEngine();
        }
    }
    /**
     * read file
     *
     * @param filePath
     * @return if file not exist, return null, else return content of file
     * @throws RuntimeException if an error occurs while operator BufferedReader
     */
    private static byte[] readFileBytes(String filePath) {
        File file = new File(filePath);
        if (file == null || !file.isFile()) {
            return null;
        }

        InputStream inputStream = null;
        ByteArrayOutputStream baos = null;
        try {
            inputStream = new FileInputStream(file);
            baos = new ByteArrayOutputStream();

            byte[] buffer = new byte[1024];
            int byteCount = 0;
            while ((byteCount = inputStream.read(buffer)) != -1) {// 循环从输入流读取
                // buffer字节
                baos.write(buffer, 0, byteCount);// 将读取的输入流写入到输出流
            }
            baos.flush();// 刷新缓冲区
            return baos.toByteArray();
        } catch (IOException e) {
            return null;
            //throw new RuntimeException("IOException occurred. ", e);
        } finally {
            close(inputStream);
            close(baos);
        }
    }

    private static void close(Closeable closeable) {
        if (closeable != null) {
            try {
                closeable.close();
            } catch (IOException e) {
                throw new RuntimeException("IOException occurred. ", e);
            }
        }
    }
}
